# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2004-2006,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# Portions Copyright (c) 2000 Fourthought Inc, USA.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Cedar Backup, release 3
# Purpose  : Provides general XML-related functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Provides general XML-related functionality.

What I'm trying to do here is abstract much of the functionality that directly
accesses the DOM tree.  This is not so much to "protect" the other code from
the DOM, but to standardize the way it's used.  It will also help extension
authors write code that easily looks more like the rest of Cedar Backup.

Module Attributes
=================

Attributes:
   TRUE_BOOLEAN_VALUES: List of boolean values in XML representing ``True``
   FALSE_BOOLEAN_VALUES: List of boolean values in XML representing ``False``
   VALID_BOOLEAN_VALUES: List of valid boolean values in XML

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""
# pylint: disable=C0111,C0103,W0511,W0104,W0106

########################################################################
# Imported modules
########################################################################

import logging
import re
import sys
from io import StringIO
from xml.dom.minidom import Node, getDOMImplementation, parseString
from xml.parsers.expat import ExpatError

########################################################################
# Module-wide constants and variables
########################################################################

logger = logging.getLogger("CedarBackup3.log.xml")

TRUE_BOOLEAN_VALUES = ["Y", "y"]
FALSE_BOOLEAN_VALUES = ["N", "n"]
VALID_BOOLEAN_VALUES = TRUE_BOOLEAN_VALUES + FALSE_BOOLEAN_VALUES


########################################################################
# Functions for creating and parsing DOM trees
########################################################################


def createInputDom(xmlData, name="cb_config"):
    """
    Creates a DOM tree based on reading an XML string.
    Returns:
        Tuple (xmlDom, parentNode) for the parsed document
    Raises:
       ValueError: If the document can't be parsed
    """
    try:
        xmlDom = parseString(xmlData)
        parentNode = readFirstChild(xmlDom, name)
        return (xmlDom, parentNode)
    except (IOError, ExpatError) as e:
        raise ValueError("Unable to parse XML document: %s" % e)


def createOutputDom(name="cb_config"):
    """
    Creates a DOM tree used for writing an XML document.
    Args:
       name: Base name of the document (root node name)
    Returns:
        Tuple (xmlDom, parentNode) for the new document
    """
    impl = getDOMImplementation()
    xmlDom = impl.createDocument(None, name, None)
    return (xmlDom, xmlDom.documentElement)


########################################################################
# Functions for reading values out of XML documents
########################################################################


def isElement(node):
    """
    Returns True or False depending on whether the XML node is an element node.
    """
    return node.nodeType == Node.ELEMENT_NODE


def readChildren(parent, name):
    """
    Returns a list of nodes with a given name immediately beneath the
    parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    Underneath, we use the Python ``getElementsByTagName`` method, which is
    pretty cool, but which (surprisingly?) returns a list of all children
    with a given name below the parent, at any level.  We just prune that
    list to include only children whose ``parentNode`` matches the passed-in
    parent.

    Args:
       parent: Parent node to search beneath
       name: Name of nodes to search for

    Returns:
        List of child nodes with correct parent, or an empty list if
    no matching nodes are found.
    """
    lst = []
    if parent is not None:
        result = parent.getElementsByTagName(name)
        for entry in result:
            if entry.parentNode is parent:
                lst.append(entry)
    return lst


def readFirstChild(parent, name):
    """
    Returns the first child with a given name immediately beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        First properly-named child of parent, or ``None`` if no matching nodes are found
    """
    result = readChildren(parent, name)
    if not result:
        return None
    return result[0]


def readStringList(parent, name):
    """
    Returns a list of the string contents associated with nodes with a given
    name immediately beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    First, we find all of the nodes using :any:`readChildren`, and then we
    retrieve the "string contents" of each of those nodes.  The returned list
    has one entry per matching node.  We assume that string contents of a
    given node belong to the first ``TEXT_NODE`` child of that node.  Nodes
    which have no ``TEXT_NODE`` children are not represented in the returned
    list.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        List of strings as described above, or ``None`` if no matching nodes are found
    """
    lst = []
    result = readChildren(parent, name)
    for entry in result:
        if entry.hasChildNodes():
            for child in entry.childNodes:
                if child.nodeType == Node.TEXT_NODE:
                    lst.append(child.nodeValue)
                    break
    if not lst:
        lst = None
    return lst


def readString(parent, name):
    """
    Returns string contents of the first child with a given name immediately
    beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.  We assume that string
    contents of a given node belong to the first ``TEXT_NODE`` child of that
    node.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        String contents of node or ``None`` if no matching nodes are found
    """
    result = readStringList(parent, name)
    if result is None:
        return None
    return result[0]


def readInteger(parent, name):
    """
    Returns integer contents of the first child with a given name immediately
    beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        Integer contents of node or ``None`` if no matching nodes are found
    Raises:
       ValueError: If the string at the location can't be converted to an integer
    """
    result = readString(parent, name)
    if result is None:
        return None
    else:
        return int(result)


def readLong(parent, name):
    """
    Returns long integer contents of the first child with a given name immediately
    beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        Long integer contents of node or ``None`` if no matching nodes are found
    Raises:
       ValueError: If the string at the location can't be converted to an integer
    """
    result = readString(parent, name)
    if result is None:
        return None
    else:
        return int(result)


def readFloat(parent, name):
    """
    Returns float contents of the first child with a given name immediately
    beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        Float contents of node or ``None`` if no matching nodes are found
    Raises:
       ValueError: If the string at the location can't be converted to a
    float value.
    """
    result = readString(parent, name)
    if result is None:
        return None
    else:
        return float(result)


def readBoolean(parent, name):
    """
    Returns boolean contents of the first child with a given name immediately
    beneath the parent.

    By "immediately beneath" the parent, we mean from among nodes that are
    direct children of the passed-in parent node.

    The string value of the node must be one of the values in :any:`VALID_BOOLEAN_VALUES`.

    Args:
       parent: Parent node to search beneath
       name: Name of node to search for

    Returns:
        Boolean contents of node or ``None`` if no matching nodes are found
    Raises:
       ValueError: If the string at the location can't be converted to a boolean
    """
    result = readString(parent, name)
    if result is None:
        return None
    else:
        if result in TRUE_BOOLEAN_VALUES:
            return True
        elif result in FALSE_BOOLEAN_VALUES:
            return False
        else:
            raise ValueError("Boolean values must be one of %s." % VALID_BOOLEAN_VALUES)


########################################################################
# Functions for writing values into XML documents
########################################################################


def addContainerNode(xmlDom, parentNode, nodeName):
    """
    Adds a container node as the next child of a parent node.

    Args:
       xmlDom: DOM tree as from ``impl.createDocument()``
       parentNode: Parent node to create child for
       nodeName: Name of the new container node

    Returns:
        Reference to the newly-created node
    """
    containerNode = xmlDom.createElement(nodeName)
    parentNode.appendChild(containerNode)
    return containerNode


def addStringNode(xmlDom, parentNode, nodeName, nodeValue):
    """
    Adds a text node as the next child of a parent, to contain a string.

    If the ``nodeValue`` is None, then the node will be created, but will be
    empty (i.e. will contain no text node child).

    Args:
       xmlDom: DOM tree as from ``impl.createDocument()``
       parentNode: Parent node to create child for
       nodeName: Name of the new container node
       nodeValue: The value to put into the node

    Returns:
        Reference to the newly-created node
    """
    containerNode = addContainerNode(xmlDom, parentNode, nodeName)
    if nodeValue is not None:
        textNode = xmlDom.createTextNode(nodeValue)
        containerNode.appendChild(textNode)
    return containerNode


def addIntegerNode(xmlDom, parentNode, nodeName, nodeValue):
    """
    Adds a text node as the next child of a parent, to contain an integer.

    If the ``nodeValue`` is None, then the node will be created, but will be
    empty (i.e. will contain no text node child).

    The integer will be converted to a string using "%d".  The result will be
    added to the document via :any:`addStringNode`.

    Args:
       xmlDom: DOM tree as from ``impl.createDocument()``
       parentNode: Parent node to create child for
       nodeName: Name of the new container node
       nodeValue: The value to put into the node

    Returns:
        Reference to the newly-created node
    """
    if nodeValue is None:
        return addStringNode(xmlDom, parentNode, nodeName, None)
    else:
        return addStringNode(xmlDom, parentNode, nodeName, "%d" % nodeValue)  # %d works for both int and long


def addLongNode(xmlDom, parentNode, nodeName, nodeValue):
    """
    Adds a text node as the next child of a parent, to contain a long integer.

    If the ``nodeValue`` is None, then the node will be created, but will be
    empty (i.e. will contain no text node child).

    The integer will be converted to a string using "%d".  The result will be
    added to the document via :any:`addStringNode`.

    Args:
       xmlDom: DOM tree as from ``impl.createDocument()``
       parentNode: Parent node to create child for
       nodeName: Name of the new container node
       nodeValue: The value to put into the node

    Returns:
        Reference to the newly-created node
    """
    if nodeValue is None:
        return addStringNode(xmlDom, parentNode, nodeName, None)
    else:
        return addStringNode(xmlDom, parentNode, nodeName, "%d" % nodeValue)  # %d works for both int and long


def addBooleanNode(xmlDom, parentNode, nodeName, nodeValue):
    """
    Adds a text node as the next child of a parent, to contain a boolean.

    If the ``nodeValue`` is None, then the node will be created, but will be
    empty (i.e. will contain no text node child).

    Boolean ``True``, or anything else interpreted as ``True`` by Python, will
    be converted to a string "Y".  Anything else will be converted to a
    string "N".  The result is added to the document via :any:`addStringNode`.

    Args:
       xmlDom: DOM tree as from ``impl.createDocument()``
       parentNode: Parent node to create child for
       nodeName: Name of the new container node
       nodeValue: The value to put into the node

    Returns:
        Reference to the newly-created node
    """
    if nodeValue is None:
        return addStringNode(xmlDom, parentNode, nodeName, None)
    else:
        if nodeValue:
            return addStringNode(xmlDom, parentNode, nodeName, "Y")
        else:
            return addStringNode(xmlDom, parentNode, nodeName, "N")


########################################################################
# Functions for serializing DOM trees
########################################################################


def serializeDom(xmlDom, indent=3):
    """
    Serializes a DOM tree and returns the result in a string.
    Args:
       xmlDom: XML DOM tree to serialize
       indent: Number of spaces to indent, as an integer
    Returns:
        String form of DOM tree, pretty-printed
    """
    xmlBuffer = StringIO()
    serializer = Serializer(xmlBuffer, "UTF-8", indent=indent)
    serializer.serialize(xmlDom)
    xmlData = xmlBuffer.getvalue()
    xmlBuffer.close()
    return xmlData


class Serializer(object):
    """
    XML serializer class.

    This is a customized serializer that I hacked together based on what I found
    in the PyXML distribution.  Basically, around release 2.7.0, the only reason
    I still had around a dependency on PyXML was for the PrettyPrint
    functionality, and that seemed pointless.  So, I stripped the PrettyPrint
    code out of PyXML and hacked bits of it off until it did just what I needed
    and no more.

    This code started out being called PrintVisitor, but I decided it makes more
    sense just calling it a serializer.  I've made nearly all of the methods
    private, and I've added a new high-level serialize() method rather than
    having clients call ``visit()``.

    Anyway, as a consequence of my hacking with it, this can't quite be called a
    complete XML serializer any more.  I ripped out support for HTML and XHTML,
    and there is also no longer any support for namespaces (which I took out
    because this dragged along a lot of extra code, and Cedar Backup doesn't use
    namespaces).  However, everything else should pretty much work as expected.

    @copyright: This code, prior to customization, was part of the PyXML
    codebase, and before that was part of the 4DOM suite developed by
    Fourthought, Inc.  It its original form, it was Copyright (c) 2000
    Fourthought Inc, USA; All Rights Reserved.
    """

    def __init__(self, stream=sys.stdout, encoding="UTF-8", indent=3):
        """
        Initialize a serializer.
        Args:
           stream: Stream to write output to
           encoding: Output encoding
           indent: Number of spaces to indent, as an integer
        """
        self.stream = stream
        self.encoding = encoding
        self._indent = indent * " "
        self._depth = 0
        self._inText = 0

    def serialize(self, xmlDom):
        """
        Serialize the passed-in XML document.
        Args:
           xmlDom: XML DOM tree to serialize
        Raises:
           ValueError: If there's an unknown node type in the document
        """
        self._visit(xmlDom)
        self.stream.write("\n")

    def _write(self, text):
        obj = _encodeText(text, self.encoding)
        self.stream.write(obj)
        return

    def _tryIndent(self):
        if not self._inText and self._indent:
            self._write("\n" + self._indent * self._depth)
        return

    def _visit(self, node):
        """
        Raises:
           ValueError: If there's an unknown node type in the document
        """
        if node.nodeType == Node.ELEMENT_NODE:
            return self._visitElement(node)

        elif node.nodeType == Node.ATTRIBUTE_NODE:
            return self._visitAttr(node)

        elif node.nodeType == Node.TEXT_NODE:
            return self._visitText(node)

        elif node.nodeType == Node.CDATA_SECTION_NODE:
            return self._visitCDATASection(node)

        elif node.nodeType == Node.ENTITY_REFERENCE_NODE:
            return self._visitEntityReference(node)

        elif node.nodeType == Node.ENTITY_NODE:
            return self._visitEntity(node)

        elif node.nodeType == Node.PROCESSING_INSTRUCTION_NODE:
            return self._visitProcessingInstruction(node)

        elif node.nodeType == Node.COMMENT_NODE:
            return self._visitComment(node)

        elif node.nodeType == Node.DOCUMENT_NODE:
            return self._visitDocument(node)

        elif node.nodeType == Node.DOCUMENT_TYPE_NODE:
            return self._visitDocumentType(node)

        elif node.nodeType == Node.DOCUMENT_FRAGMENT_NODE:
            return self._visitDocumentFragment(node)

        elif node.nodeType == Node.NOTATION_NODE:
            return self._visitNotation(node)

        # It has a node type, but we don't know how to handle it
        raise ValueError("Unknown node type: %s" % repr(node))

    def _visitNodeList(self, node, exclude=None):
        for curr in node:
            curr is not exclude and self._visit(curr)
        return

    def _visitNamedNodeMap(self, node):
        for item in list(node.values()):
            self._visit(item)
        return

    def _visitAttr(self, node):
        self._write(" " + node.name)
        value = node.value
        text = _translateCDATA(value, self.encoding)
        text, delimiter = _translateCDATAAttr(text)
        self.stream.write("=%s%s%s" % (delimiter, text, delimiter))
        return

    def _visitProlog(self):
        self._write("<?xml version='1.0' encoding='%s'?>" % (self.encoding or "utf-8"))
        self._inText = 0
        return

    def _visitDocument(self, node):
        self._visitProlog()
        node.doctype and self._visitDocumentType(node.doctype)
        self._visitNodeList(node.childNodes, exclude=node.doctype)
        return

    def _visitDocumentFragment(self, node):
        self._visitNodeList(node.childNodes)
        return

    def _visitElement(self, node):
        self._tryIndent()
        self._write("<%s" % node.tagName)
        for attr in list(node.attributes.values()):
            self._visitAttr(attr)
        if len(node.childNodes):
            self._write(">")
            self._depth = self._depth + 1
            self._visitNodeList(node.childNodes)
            self._depth = self._depth - 1
            not (self._inText) and self._tryIndent()
            self._write("</%s>" % node.tagName)
        else:
            self._write("/>")
        self._inText = 0
        return

    def _visitText(self, node):
        text = node.data
        if self._indent:
            text.strip()
        if text:
            text = _translateCDATA(text, self.encoding)
            self.stream.write(text)
            self._inText = 1
        return

    def _visitDocumentType(self, doctype):
        if not doctype.systemId and not doctype.publicId:
            return
        self._tryIndent()
        self._write("<!DOCTYPE %s" % doctype.name)
        if doctype.systemId and '"' in doctype.systemId:
            system = "'%s'" % doctype.systemId
        else:
            system = '"%s"' % doctype.systemId
        if doctype.publicId and '"' in doctype.publicId:
            # We should probably throw an error
            # Valid characters:  <space> | <newline> | <linefeed> |
            #                    [a-zA-Z0-9] | [-'()+,./:=?;!*#@$_%]
            public = "'%s'" % doctype.publicId
        else:
            public = '"%s"' % doctype.publicId
        if doctype.publicId and doctype.systemId:
            self._write(" PUBLIC %s %s" % (public, system))
        elif doctype.systemId:
            self._write(" SYSTEM %s" % system)
        if doctype.entities or doctype.notations:
            self._write(" [")
            self._depth = self._depth + 1
            self._visitNamedNodeMap(doctype.entities)
            self._visitNamedNodeMap(doctype.notations)
            self._depth = self._depth - 1
            self._tryIndent()
            self._write("]>")
        else:
            self._write(">")
        self._inText = 0
        return

    def _visitEntity(self, node):
        """Visited from a NamedNodeMap in DocumentType"""
        self._tryIndent()
        self._write("<!ENTITY %s" % (node.nodeName))
        node.publicId and self._write(" PUBLIC %s" % node.publicId)
        node.systemId and self._write(" SYSTEM %s" % node.systemId)
        node.notationName and self._write(" NDATA %s" % node.notationName)
        self._write(">")
        return

    def _visitNotation(self, node):
        """Visited from a NamedNodeMap in DocumentType"""
        self._tryIndent()
        self._write("<!NOTATION %s" % node.nodeName)
        node.publicId and self._write(" PUBLIC %s" % node.publicId)
        node.systemId and self._write(" SYSTEM %s" % node.systemId)
        self._write(">")
        return

    def _visitCDATASection(self, node):
        self._tryIndent()
        self._write("<![CDATA[%s]]>" % (node.data))
        self._inText = 0
        return

    def _visitComment(self, node):
        self._tryIndent()
        self._write("<!--%s-->" % (node.data))
        self._inText = 0
        return

    def _visitEntityReference(self, node):
        self._write("&%s;" % node.nodeName)
        self._inText = 1
        return

    def _visitProcessingInstruction(self, node):
        self._tryIndent()
        self._write("<?%s %s?>" % (node.target, node.data))
        self._inText = 0
        return


# pylint: disable=W0613
def _encodeText(text, encoding):
    """Safely encodes the passed-in text as a Unicode string, converting bytes to UTF-8 if necessary."""
    if text is None:
        return text
    try:
        if isinstance(text, bytes):
            text = str(text, "utf-8")
        return text
    except UnicodeError:
        raise ValueError("Path could not be safely encoded as utf-8.")


def _translateCDATAAttr(characters):
    """
    Handles normalization and some intelligence about quoting.

    @copyright: This code, prior to customization, was part of the PyXML
    codebase, and before that was part of the 4DOM suite developed by
    Fourthought, Inc.  It its original form, it was Copyright (c) 2000
    Fourthought Inc, USA; All Rights Reserved.
    """
    if not characters:
        return "", "'"
    if "'" in characters:
        delimiter = '"'
        new_chars = re.sub('"', "&quot;", characters)
    else:
        delimiter = "'"
        new_chars = re.sub("'", "&apos;", characters)
    # FIXME: There's more to normalization
    # Convert attribute new-lines to character entity
    # characters is possibly shorter than new_chars (no entities)
    if "\n" in characters:
        new_chars = re.sub("\n", "&#10;", new_chars)
    return new_chars, delimiter


# Note: Unicode object only for now
def _translateCDATA(characters, encoding="UTF-8", prev_chars="", markupSafe=0):
    """
    @copyright: This code, prior to customization, was part of the PyXML
    codebase, and before that was part of the 4DOM suite developed by
    Fourthought, Inc.  It its original form, it was Copyright (c) 2000
    Fourthought Inc, USA; All Rights Reserved.
    """
    CDATA_CHAR_PATTERN = re.compile("[&<]|]]>")
    CHAR_TO_ENTITY = {"&": "&amp;", "<": "&lt;", "]]>": "]]&gt;"}
    ILLEGAL_LOW_CHARS = "[\x01-\x08\x0B-\x0C\x0E-\x1F]"
    ILLEGAL_HIGH_CHARS = "\xEF\xBF[\xBE\xBF]"
    XML_ILLEGAL_CHAR_PATTERN = re.compile("%s|%s" % (ILLEGAL_LOW_CHARS, ILLEGAL_HIGH_CHARS))
    if not characters:
        return ""
    if not markupSafe:
        if CDATA_CHAR_PATTERN.search(characters):
            new_string = CDATA_CHAR_PATTERN.subn(lambda m, d=CHAR_TO_ENTITY: d[m.group()], characters)[0]
        else:
            new_string = characters
        if prev_chars[-2:] == "]]" and characters[0] == ">":
            new_string = "&gt;" + new_string[1:]
    else:
        new_string = characters
    # Note: use decimal char entity rep because some browsers are broken
    # FIXME: This will bomb for high characters.  Should, for instance, detect
    # The UTF-8 for 0xFFFE and put out &#xFFFE;
    if XML_ILLEGAL_CHAR_PATTERN.search(new_string):
        new_string = XML_ILLEGAL_CHAR_PATTERN.subn(lambda m: "&#%i;" % ord(m.group()), new_string)[0]
    new_string = _encodeText(new_string, encoding)
    return new_string



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2006,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Cedar Backup, release 3
# Purpose  : Tests mbox extension functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Unit tests for CedarBackup3/extend/mbox.py.

Code Coverage
=============

   This module contains individual tests for the many of the public functions
   and classes implemented in extend/mbox.py.  There are also tests for
   several of the private methods.

Naming Conventions
==================

   I prefer to avoid large unit tests which validate more than one piece of
   functionality, and I prefer to avoid using overly descriptive (read: long)
   test names, as well.  Instead, I use lots of very small tests that each
   validate one specific thing.  These small tests are then named with an index
   number, yielding something like ``testAddDir_001`` or ``testValidate_010``.
   Each method has a docstring describing what it's supposed to accomplish.  I
   feel that this makes it easier to judge how important a given failure is,
   and also makes it somewhat easier to diagnose and fix individual problems.

Testing XML Extraction
======================

   It's difficult to validated that generated XML is exactly "right",
   especially when dealing with pretty-printed XML.  We can't just provide a
   constant string and say "the result must match this".  Instead, what we do
   is extract a node, build some XML from it, and then feed that XML back into
   another object's constructor.  If that parse process succeeds and the old
   object is equal to the new object, we assume that the extract was
   successful.

   It would arguably be better if we could do a completely independent check -
   but implementing that check would be equivalent to re-implementing all of
   the existing functionality that we're validating here!  After all, the most
   important thing is that data can move seamlessly from object to XML document
   and back to object.

Full vs. Reduced Tests
======================

   All of the tests in this module are considered safe to be run in an average
   build environment.  There is a no need to use a MBOXTESTS_FULL environment
   variable to provide a "reduced feature set" test suite as for some of the
   other test modules.

@author Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Import modules and do runtime validations
########################################################################

import unittest

from CedarBackup3.extend.mbox import LocalConfig, MboxConfig, MboxDir, MboxFile
from CedarBackup3.testutil import configureLogging, failUnlessAssignRaises, findResources
from CedarBackup3.xmlutil import createOutputDom, serializeDom

#######################################################################
# Module-wide configuration and constants
#######################################################################

DATA_DIRS = [
    "./data",
    "./tests/data",
]
RESOURCES = [
    "mbox.conf.1",
    "mbox.conf.2",
    "mbox.conf.3",
    "mbox.conf.4",
]


#######################################################################
# Test Case Classes
#######################################################################

#####################
# TestMboxFile class
#####################


class TestMboxFile(unittest.TestCase):
    """Tests for the MboxFile class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
        Just make sure that the string functions don't have errors (i.e. bad variable names).
        """
        obj = MboxFile()
        obj.__repr__()
        obj.__str__()

    ##################################
    # Test constructor and attributes
    ##################################

    def testConstructor_001(self):
        """
        Test constructor with no values filled in.
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.absolutePath)
        self.assertEqual(None, mboxFile.collectMode)
        self.assertEqual(None, mboxFile.compressMode)

    def testConstructor_002(self):
        """
        Test constructor with all values filled in.
        """
        mboxFile = MboxFile("/path/to/it", "daily", "gzip")
        self.assertEqual("/path/to/it", mboxFile.absolutePath)
        self.assertEqual("daily", mboxFile.collectMode)
        self.assertEqual("gzip", mboxFile.compressMode)

    def testConstructor_003(self):
        """
        Test assignment of absolutePath attribute, None value.
        """
        mboxFile = MboxFile(absolutePath="/path/to/something")
        self.assertEqual("/path/to/something", mboxFile.absolutePath)
        mboxFile.absolutePath = None
        self.assertEqual(None, mboxFile.absolutePath)

    def testConstructor_004(self):
        """
        Test assignment of absolutePath attribute, valid value.
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.absolutePath)
        mboxFile.absolutePath = "/path/to/whatever"
        self.assertEqual("/path/to/whatever", mboxFile.absolutePath)

    def testConstructor_005(self):
        """
        Test assignment of absolutePath attribute, invalid value (empty).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.absolutePath)
        self.failUnlessAssignRaises(ValueError, mboxFile, "absolutePath", "")
        self.assertEqual(None, mboxFile.absolutePath)

    def testConstructor_006(self):
        """
        Test assignment of absolutePath attribute, invalid value (not absolute).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.absolutePath)
        self.failUnlessAssignRaises(ValueError, mboxFile, "absolutePath", "relative/path")
        self.assertEqual(None, mboxFile.absolutePath)

    def testConstructor_007(self):
        """
        Test assignment of collectMode attribute, None value.
        """
        mboxFile = MboxFile(collectMode="daily")
        self.assertEqual("daily", mboxFile.collectMode)
        mboxFile.collectMode = None
        self.assertEqual(None, mboxFile.collectMode)

    def testConstructor_008(self):
        """
        Test assignment of collectMode attribute, valid value.
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.collectMode)
        mboxFile.collectMode = "daily"
        self.assertEqual("daily", mboxFile.collectMode)
        mboxFile.collectMode = "weekly"
        self.assertEqual("weekly", mboxFile.collectMode)
        mboxFile.collectMode = "incr"
        self.assertEqual("incr", mboxFile.collectMode)

    def testConstructor_009(self):
        """
        Test assignment of collectMode attribute, invalid value (empty).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.collectMode)
        self.failUnlessAssignRaises(ValueError, mboxFile, "collectMode", "")
        self.assertEqual(None, mboxFile.collectMode)

    def testConstructor_010(self):
        """
        Test assignment of collectMode attribute, invalid value (not in list).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.collectMode)
        self.failUnlessAssignRaises(ValueError, mboxFile, "collectMode", "monthly")
        self.assertEqual(None, mboxFile.collectMode)

    def testConstructor_011(self):
        """
        Test assignment of compressMode attribute, None value.
        """
        mboxFile = MboxFile(compressMode="gzip")
        self.assertEqual("gzip", mboxFile.compressMode)
        mboxFile.compressMode = None
        self.assertEqual(None, mboxFile.compressMode)

    def testConstructor_012(self):
        """
        Test assignment of compressMode attribute, valid value.
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.compressMode)
        mboxFile.compressMode = "none"
        self.assertEqual("none", mboxFile.compressMode)
        mboxFile.compressMode = "bzip2"
        self.assertEqual("bzip2", mboxFile.compressMode)
        mboxFile.compressMode = "gzip"
        self.assertEqual("gzip", mboxFile.compressMode)

    def testConstructor_013(self):
        """
        Test assignment of compressMode attribute, invalid value (empty).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.compressMode)
        self.failUnlessAssignRaises(ValueError, mboxFile, "compressMode", "")
        self.assertEqual(None, mboxFile.compressMode)

    def testConstructor_014(self):
        """
        Test assignment of compressMode attribute, invalid value (not in list).
        """
        mboxFile = MboxFile()
        self.assertEqual(None, mboxFile.compressMode)
        self.failUnlessAssignRaises(ValueError, mboxFile, "compressMode", "compress")
        self.assertEqual(None, mboxFile.compressMode)

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
        Test comparison of two identical objects, all attributes None.
        """
        mboxFile1 = MboxFile()
        mboxFile2 = MboxFile()
        self.assertEqual(mboxFile1, mboxFile2)
        self.assertTrue(mboxFile1 == mboxFile2)
        self.assertTrue(not mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(mboxFile1 >= mboxFile2)
        self.assertTrue(not mboxFile1 != mboxFile2)

    def testComparison_002(self):
        """
        Test comparison of two identical objects, all attributes non-None.
        """
        mboxFile1 = MboxFile("/path", "daily", "gzip")
        mboxFile2 = MboxFile("/path", "daily", "gzip")
        self.assertEqual(mboxFile1, mboxFile2)
        self.assertTrue(mboxFile1 == mboxFile2)
        self.assertTrue(not mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(mboxFile1 >= mboxFile2)
        self.assertTrue(not mboxFile1 != mboxFile2)

    def testComparison_003(self):
        """
        Test comparison of two differing objects, absolutePath differs (one None).
        """
        mboxFile1 = MboxFile()
        mboxFile2 = MboxFile(absolutePath="/zippy")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)

    def testComparison_004(self):
        """
        Test comparison of two differing objects, absolutePath differs.
        """
        mboxFile1 = MboxFile("/path", "daily", "gzip")
        mboxFile2 = MboxFile("/zippy", "daily", "gzip")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)

    def testComparison_005(self):
        """
        Test comparison of two differing objects, collectMode differs (one None).
        """
        mboxFile1 = MboxFile()
        mboxFile2 = MboxFile(collectMode="incr")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)

    def testComparison_006(self):
        """
        Test comparison of two differing objects, collectMode differs.
        """
        mboxFile1 = MboxFile("/path", "daily", "gzip")
        mboxFile2 = MboxFile("/path", "incr", "gzip")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)

    def testComparison_007(self):
        """
        Test comparison of two differing objects, compressMode differs (one None).
        """
        mboxFile1 = MboxFile()
        mboxFile2 = MboxFile(compressMode="gzip")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)

    def testComparison_008(self):
        """
        Test comparison of two differing objects, compressMode differs.
        """
        mboxFile1 = MboxFile("/path", "daily", "bzip2")
        mboxFile2 = MboxFile("/path", "daily", "gzip")
        self.assertNotEqual(mboxFile1, mboxFile2)
        self.assertTrue(not mboxFile1 == mboxFile2)
        self.assertTrue(mboxFile1 < mboxFile2)
        self.assertTrue(mboxFile1 <= mboxFile2)
        self.assertTrue(not mboxFile1 > mboxFile2)
        self.assertTrue(not mboxFile1 >= mboxFile2)
        self.assertTrue(mboxFile1 != mboxFile2)


#####################
# TestMboxDir class
#####################


class TestMboxDir(unittest.TestCase):
    """Tests for the MboxDir class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
        Just make sure that the string functions don't have errors (i.e. bad variable names).
        """
        obj = MboxDir()
        obj.__repr__()
        obj.__str__()

    ##################################
    # Test constructor and attributes
    ##################################

    def testConstructor_001(self):
        """
        Test constructor with no values filled in.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.absolutePath)
        self.assertEqual(None, mboxDir.collectMode)
        self.assertEqual(None, mboxDir.compressMode)
        self.assertEqual(None, mboxDir.relativeExcludePaths)
        self.assertEqual(None, mboxDir.excludePatterns)

    def testConstructor_002(self):
        """
        Test constructor with all values filled in.
        """
        mboxDir = MboxDir("/path/to/it", "daily", "gzip", ["whatever"], [".*SPAM.*"])
        self.assertEqual("/path/to/it", mboxDir.absolutePath)
        self.assertEqual("daily", mboxDir.collectMode)
        self.assertEqual("gzip", mboxDir.compressMode)
        self.assertEqual(["whatever"], mboxDir.relativeExcludePaths)
        self.assertEqual([".*SPAM.*"], mboxDir.excludePatterns)

    def testConstructor_003(self):
        """
        Test assignment of absolutePath attribute, None value.
        """
        mboxDir = MboxDir(absolutePath="/path/to/something")
        self.assertEqual("/path/to/something", mboxDir.absolutePath)
        mboxDir.absolutePath = None
        self.assertEqual(None, mboxDir.absolutePath)

    def testConstructor_004(self):
        """
        Test assignment of absolutePath attribute, valid value.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.absolutePath)
        mboxDir.absolutePath = "/path/to/whatever"
        self.assertEqual("/path/to/whatever", mboxDir.absolutePath)

    def testConstructor_005(self):
        """
        Test assignment of absolutePath attribute, invalid value (empty).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.absolutePath)
        self.failUnlessAssignRaises(ValueError, mboxDir, "absolutePath", "")
        self.assertEqual(None, mboxDir.absolutePath)

    def testConstructor_006(self):
        """
        Test assignment of absolutePath attribute, invalid value (not absolute).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.absolutePath)
        self.failUnlessAssignRaises(ValueError, mboxDir, "absolutePath", "relative/path")
        self.assertEqual(None, mboxDir.absolutePath)

    def testConstructor_007(self):
        """
        Test assignment of collectMode attribute, None value.
        """
        mboxDir = MboxDir(collectMode="daily")
        self.assertEqual("daily", mboxDir.collectMode)
        mboxDir.collectMode = None
        self.assertEqual(None, mboxDir.collectMode)

    def testConstructor_008(self):
        """
        Test assignment of collectMode attribute, valid value.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.collectMode)
        mboxDir.collectMode = "daily"
        self.assertEqual("daily", mboxDir.collectMode)
        mboxDir.collectMode = "weekly"
        self.assertEqual("weekly", mboxDir.collectMode)
        mboxDir.collectMode = "incr"
        self.assertEqual("incr", mboxDir.collectMode)

    def testConstructor_009(self):
        """
        Test assignment of collectMode attribute, invalid value (empty).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.collectMode)
        self.failUnlessAssignRaises(ValueError, mboxDir, "collectMode", "")
        self.assertEqual(None, mboxDir.collectMode)

    def testConstructor_010(self):
        """
        Test assignment of collectMode attribute, invalid value (not in list).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.collectMode)
        self.failUnlessAssignRaises(ValueError, mboxDir, "collectMode", "monthly")
        self.assertEqual(None, mboxDir.collectMode)

    def testConstructor_011(self):
        """
        Test assignment of compressMode attribute, None value.
        """
        mboxDir = MboxDir(compressMode="gzip")
        self.assertEqual("gzip", mboxDir.compressMode)
        mboxDir.compressMode = None
        self.assertEqual(None, mboxDir.compressMode)

    def testConstructor_012(self):
        """
        Test assignment of compressMode attribute, valid value.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.compressMode)
        mboxDir.compressMode = "none"
        self.assertEqual("none", mboxDir.compressMode)
        mboxDir.compressMode = "bzip2"
        self.assertEqual("bzip2", mboxDir.compressMode)
        mboxDir.compressMode = "gzip"
        self.assertEqual("gzip", mboxDir.compressMode)

    def testConstructor_013(self):
        """
        Test assignment of compressMode attribute, invalid value (empty).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.compressMode)
        self.failUnlessAssignRaises(ValueError, mboxDir, "compressMode", "")
        self.assertEqual(None, mboxDir.compressMode)

    def testConstructor_014(self):
        """
        Test assignment of compressMode attribute, invalid value (not in list).
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.compressMode)
        self.failUnlessAssignRaises(ValueError, mboxDir, "compressMode", "compress")
        self.assertEqual(None, mboxDir.compressMode)

    def testConstructor_015(self):
        """
        Test assignment of relativeExcludePaths attribute, None value.
        """
        mboxDir = MboxDir(relativeExcludePaths=[])
        self.assertEqual([], mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths = None
        self.assertEqual(None, mboxDir.relativeExcludePaths)

    def testConstructor_016(self):
        """
        Test assignment of relativeExcludePaths attribute, [] value.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths = []
        self.assertEqual([], mboxDir.relativeExcludePaths)

    def testConstructor_017(self):
        """
        Test assignment of relativeExcludePaths attribute, single valid entry.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths = [
            "stuff",
        ]
        self.assertEqual(["stuff"], mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths.insert(0, "bogus")
        self.assertEqual(["bogus", "stuff"], mboxDir.relativeExcludePaths)

    def testConstructor_018(self):
        """
        Test assignment of relativeExcludePaths attribute, multiple valid
        entries.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths = [
            "bogus",
            "stuff",
        ]
        self.assertEqual(["bogus", "stuff"], mboxDir.relativeExcludePaths)
        mboxDir.relativeExcludePaths.append("more")
        self.assertEqual(["bogus", "stuff", "more"], mboxDir.relativeExcludePaths)

    def testConstructor_019(self):
        """
        Test assignment of excludePatterns attribute, None value.
        """
        mboxDir = MboxDir(excludePatterns=[])
        self.assertEqual([], mboxDir.excludePatterns)
        mboxDir.excludePatterns = None
        self.assertEqual(None, mboxDir.excludePatterns)

    def testConstructor_020(self):
        """
        Test assignment of excludePatterns attribute, [] value.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        mboxDir.excludePatterns = []
        self.assertEqual([], mboxDir.excludePatterns)

    def testConstructor_021(self):
        """
        Test assignment of excludePatterns attribute, single valid entry.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        mboxDir.excludePatterns = [
            "valid",
        ]
        self.assertEqual(["valid"], mboxDir.excludePatterns)
        mboxDir.excludePatterns.append("more")
        self.assertEqual(["valid", "more"], mboxDir.excludePatterns)

    def testConstructor_022(self):
        """
        Test assignment of excludePatterns attribute, multiple valid entries.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        mboxDir.excludePatterns = [
            "valid",
            "more",
        ]
        self.assertEqual(["valid", "more"], mboxDir.excludePatterns)
        mboxDir.excludePatterns.insert(1, "bogus")
        self.assertEqual(["valid", "bogus", "more"], mboxDir.excludePatterns)

    def testConstructor_023(self):
        """
        Test assignment of excludePatterns attribute, single invalid entry.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        self.failUnlessAssignRaises(ValueError, mboxDir, "excludePatterns", ["*.jpg"])
        self.assertEqual(None, mboxDir.excludePatterns)

    def testConstructor_024(self):
        """
        Test assignment of excludePatterns attribute, multiple invalid entries.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        self.failUnlessAssignRaises(ValueError, mboxDir, "excludePatterns", ["*.jpg", "*"])
        self.assertEqual(None, mboxDir.excludePatterns)

    def testConstructor_025(self):
        """
        Test assignment of excludePatterns attribute, mixed valid and invalid
        entries.
        """
        mboxDir = MboxDir()
        self.assertEqual(None, mboxDir.excludePatterns)
        self.failUnlessAssignRaises(ValueError, mboxDir, "excludePatterns", ["*.jpg", "valid"])
        self.assertEqual(None, mboxDir.excludePatterns)

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
        Test comparison of two identical objects, all attributes None.
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir()
        self.assertEqual(mboxDir1, mboxDir2)
        self.assertTrue(mboxDir1 == mboxDir2)
        self.assertTrue(not mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(mboxDir1 >= mboxDir2)
        self.assertTrue(not mboxDir1 != mboxDir2)

    def testComparison_002(self):
        """
        Test comparison of two identical objects, all attributes non-None.
        """
        mboxDir1 = MboxDir("/path", "daily", "gzip")
        mboxDir2 = MboxDir("/path", "daily", "gzip")
        self.assertEqual(mboxDir1, mboxDir2)
        self.assertTrue(mboxDir1 == mboxDir2)
        self.assertTrue(not mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(mboxDir1 >= mboxDir2)
        self.assertTrue(not mboxDir1 != mboxDir2)

    def testComparison_003(self):
        """
        Test comparison of two differing objects, absolutePath differs (one None).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(absolutePath="/zippy")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_004(self):
        """
        Test comparison of two differing objects, absolutePath differs.
        """
        mboxDir1 = MboxDir("/path", "daily", "gzip")
        mboxDir2 = MboxDir("/zippy", "daily", "gzip")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_005(self):
        """
        Test comparison of two differing objects, collectMode differs (one None).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(collectMode="incr")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_006(self):
        """
        Test comparison of two differing objects, collectMode differs.
        """
        mboxDir1 = MboxDir("/path", "daily", "gzip")
        mboxDir2 = MboxDir("/path", "incr", "gzip")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_007(self):
        """
        Test comparison of two differing objects, compressMode differs (one None).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(compressMode="gzip")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_008(self):
        """
        Test comparison of two differing objects, compressMode differs.
        """
        mboxDir1 = MboxDir("/path", "daily", "bzip2")
        mboxDir2 = MboxDir("/path", "daily", "gzip")
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_009(self):
        """
        Test comparison of two differing objects, relativeExcludePaths differs
        (one None, one empty).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(relativeExcludePaths=[])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_010(self):
        """
        Test comparison of two differing objects, relativeExcludePaths differs
        (one None, one not empty).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(relativeExcludePaths=["stuff", "other"])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_011(self):
        """
        Test comparison of two differing objects, relativeExcludePaths differs
        (one empty, one not empty).
        """
        mboxDir1 = MboxDir("/etc/whatever", "incr", "none", ["one"], [])
        mboxDir2 = MboxDir("/etc/whatever", "incr", "none", [], [])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(not mboxDir1 < mboxDir2)
        self.assertTrue(not mboxDir1 <= mboxDir2)
        self.assertTrue(mboxDir1 > mboxDir2)
        self.assertTrue(mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_012(self):
        """
        Test comparison of two differing objects, relativeExcludePaths differs
        (both not empty).
        """
        mboxDir1 = MboxDir("/etc/whatever", "incr", "none", ["one"], [])
        mboxDir2 = MboxDir("/etc/whatever", "incr", "none", ["two"], [])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_013(self):
        """
        Test comparison of two differing objects, excludePatterns differs (one
        None, one empty).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(excludePatterns=[])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_014(self):
        """
        Test comparison of two differing objects, excludePatterns differs (one
        None, one not empty).
        """
        mboxDir1 = MboxDir()
        mboxDir2 = MboxDir(excludePatterns=["one", "two", "three"])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_015(self):
        """
        Test comparison of two differing objects, excludePatterns differs (one
        empty, one not empty).
        """
        mboxDir1 = MboxDir("/etc/whatever", "incr", "none", [], [])
        mboxDir2 = MboxDir("/etc/whatever", "incr", "none", [], ["pattern"])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)

    def testComparison_016(self):
        """
        Test comparison of two differing objects, excludePatterns differs (both
        not empty).
        """
        mboxDir1 = MboxDir("/etc/whatever", "incr", "none", [], ["p1"])
        mboxDir2 = MboxDir("/etc/whatever", "incr", "none", [], ["p2"])
        self.assertNotEqual(mboxDir1, mboxDir2)
        self.assertTrue(not mboxDir1 == mboxDir2)
        self.assertTrue(mboxDir1 < mboxDir2)
        self.assertTrue(mboxDir1 <= mboxDir2)
        self.assertTrue(not mboxDir1 > mboxDir2)
        self.assertTrue(not mboxDir1 >= mboxDir2)
        self.assertTrue(mboxDir1 != mboxDir2)


#######################
# TestMboxConfig class
#######################


class TestMboxConfig(unittest.TestCase):
    """Tests for the MboxConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
        Just make sure that the string functions don't have errors (i.e. bad variable names).
        """
        obj = MboxConfig()
        obj.__repr__()
        obj.__str__()

    ##################################
    # Test constructor and attributes
    ##################################

    def testConstructor_001(self):
        """
        Test constructor with no values filled in.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.collectMode)
        self.assertEqual(None, mbox.compressMode)
        self.assertEqual(None, mbox.mboxFiles)
        self.assertEqual(None, mbox.mboxDirs)

    def testConstructor_002(self):
        """
        Test constructor with all values filled in, with valid values, mboxFiles=None and mboxDirs=None.
        """
        mbox = MboxConfig("daily", "gzip", None, None)
        self.assertEqual("daily", mbox.collectMode)
        self.assertEqual("gzip", mbox.compressMode)
        self.assertEqual(None, mbox.mboxFiles)
        self.assertEqual(None, mbox.mboxDirs)

    def testConstructor_003(self):
        """
        Test constructor with all values filled in, with valid values, no mboxFiles, no mboxDirs.
        """
        mbox = MboxConfig("daily", "gzip", [], [])
        self.assertEqual("daily", mbox.collectMode)
        self.assertEqual("gzip", mbox.compressMode)
        self.assertEqual([], mbox.mboxFiles)
        self.assertEqual([], mbox.mboxDirs)

    def testConstructor_004(self):
        """
        Test constructor with all values filled in, with valid values, with one mboxFile, no mboxDirs.
        """
        mboxFiles = [
            MboxFile(),
        ]
        mbox = MboxConfig("daily", "gzip", mboxFiles, [])
        self.assertEqual("daily", mbox.collectMode)
        self.assertEqual("gzip", mbox.compressMode)
        self.assertEqual(mboxFiles, mbox.mboxFiles)
        self.assertEqual([], mbox.mboxDirs)

    def testConstructor_005(self):
        """
        Test constructor with all values filled in, with valid values, with no mboxFiles, one mboxDir.
        """
        mboxDirs = [
            MboxDir(),
        ]
        mbox = MboxConfig("daily", "gzip", [], mboxDirs)
        self.assertEqual("daily", mbox.collectMode)
        self.assertEqual("gzip", mbox.compressMode)
        self.assertEqual([], mbox.mboxFiles)
        self.assertEqual(mboxDirs, mbox.mboxDirs)

    def testConstructor_006(self):
        """
        Test constructor with all values filled in, with valid values, with multiple mboxFiles and mboxDirs.
        """
        mboxFiles = [
            MboxFile(collectMode="daily"),
            MboxFile(collectMode="weekly"),
        ]
        mboxDirs = [
            MboxDir(collectMode="weekly"),
            MboxDir(collectMode="incr"),
        ]
        mbox = MboxConfig("daily", "gzip", mboxFiles=mboxFiles, mboxDirs=mboxDirs)
        self.assertEqual("daily", mbox.collectMode)
        self.assertEqual("gzip", mbox.compressMode)
        self.assertEqual(mboxFiles, mbox.mboxFiles)
        self.assertEqual(mboxDirs, mbox.mboxDirs)

    def testConstructor_007(self):
        """
        Test assignment of collectMode attribute, None value.
        """
        mbox = MboxConfig(collectMode="daily")
        self.assertEqual("daily", mbox.collectMode)
        mbox.collectMode = None
        self.assertEqual(None, mbox.collectMode)

    def testConstructor_008(self):
        """
        Test assignment of collectMode attribute, valid value.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.collectMode)
        mbox.collectMode = "weekly"
        self.assertEqual("weekly", mbox.collectMode)

    def testConstructor_009(self):
        """
        Test assignment of collectMode attribute, invalid value (empty).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.collectMode)
        self.failUnlessAssignRaises(ValueError, mbox, "collectMode", "")
        self.assertEqual(None, mbox.collectMode)

    def testConstructor_010(self):
        """
        Test assignment of compressMode attribute, None value.
        """
        mbox = MboxConfig(compressMode="gzip")
        self.assertEqual("gzip", mbox.compressMode)
        mbox.compressMode = None
        self.assertEqual(None, mbox.compressMode)

    def testConstructor_011(self):
        """
        Test assignment of compressMode attribute, valid value.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.compressMode)
        mbox.compressMode = "bzip2"
        self.assertEqual("bzip2", mbox.compressMode)

    def testConstructor_012(self):
        """
        Test assignment of compressMode attribute, invalid value (empty).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.compressMode)
        self.failUnlessAssignRaises(ValueError, mbox, "compressMode", "")
        self.assertEqual(None, mbox.compressMode)

    def testConstructor_013(self):
        """
        Test assignment of mboxFiles attribute, None value.
        """
        mbox = MboxConfig(mboxFiles=[])
        self.assertEqual([], mbox.mboxFiles)
        mbox.mboxFiles = None
        self.assertEqual(None, mbox.mboxFiles)

    def testConstructor_014(self):
        """
        Test assignment of mboxFiles attribute, [] value.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        mbox.mboxFiles = []
        self.assertEqual([], mbox.mboxFiles)

    def testConstructor_015(self):
        """
        Test assignment of mboxFiles attribute, single valid entry.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        mbox.mboxFiles = [
            MboxFile(),
        ]
        self.assertEqual([MboxFile()], mbox.mboxFiles)
        mbox.mboxFiles.append(MboxFile(collectMode="daily"))
        self.assertEqual([MboxFile(), MboxFile(collectMode="daily")], mbox.mboxFiles)

    def testConstructor_016(self):
        """
        Test assignment of mboxFiles attribute, multiple valid entries.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        mbox.mboxFiles = [
            MboxFile(collectMode="daily"),
            MboxFile(collectMode="weekly"),
        ]
        self.assertEqual([MboxFile(collectMode="daily"), MboxFile(collectMode="weekly")], mbox.mboxFiles)
        mbox.mboxFiles.append(MboxFile(collectMode="incr"))
        self.assertEqual(
            [MboxFile(collectMode="daily"), MboxFile(collectMode="weekly"), MboxFile(collectMode="incr")], mbox.mboxFiles
        )

    def testConstructor_017(self):
        """
        Test assignment of mboxFiles attribute, single invalid entry (None).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxFiles", [None])
        self.assertEqual(None, mbox.mboxFiles)

    def testConstructor_018(self):
        """
        Test assignment of mboxFiles attribute, single invalid entry (wrong type).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxFiles", [MboxDir()])
        self.assertEqual(None, mbox.mboxFiles)

    def testConstructor_019(self):
        """
        Test assignment of mboxFiles attribute, mixed valid and invalid entries.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxFiles)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxFiles", [MboxFile(), MboxDir()])
        self.assertEqual(None, mbox.mboxFiles)

    def testConstructor_020(self):
        """
        Test assignment of mboxDirs attribute, None value.
        """
        mbox = MboxConfig(mboxDirs=[])
        self.assertEqual([], mbox.mboxDirs)
        mbox.mboxDirs = None
        self.assertEqual(None, mbox.mboxDirs)

    def testConstructor_021(self):
        """
        Test assignment of mboxDirs attribute, [] value.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        mbox.mboxDirs = []
        self.assertEqual([], mbox.mboxDirs)

    def testConstructor_022(self):
        """
        Test assignment of mboxDirs attribute, single valid entry.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        mbox.mboxDirs = [
            MboxDir(),
        ]
        self.assertEqual([MboxDir()], mbox.mboxDirs)
        mbox.mboxDirs.append(MboxDir(collectMode="daily"))
        self.assertEqual([MboxDir(), MboxDir(collectMode="daily")], mbox.mboxDirs)

    def testConstructor_023(self):
        """
        Test assignment of mboxDirs attribute, multiple valid entries.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        mbox.mboxDirs = [
            MboxDir(collectMode="daily"),
            MboxDir(collectMode="weekly"),
        ]
        self.assertEqual([MboxDir(collectMode="daily"), MboxDir(collectMode="weekly")], mbox.mboxDirs)
        mbox.mboxDirs.append(MboxDir(collectMode="incr"))
        self.assertEqual([MboxDir(collectMode="daily"), MboxDir(collectMode="weekly"), MboxDir(collectMode="incr")], mbox.mboxDirs)

    def testConstructor_024(self):
        """
        Test assignment of mboxDirs attribute, single invalid entry (None).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxDirs", [None])
        self.assertEqual(None, mbox.mboxDirs)

    def testConstructor_025(self):
        """
        Test assignment of mboxDirs attribute, single invalid entry (wrong type).
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxDirs", [MboxFile()])
        self.assertEqual(None, mbox.mboxDirs)

    def testConstructor_026(self):
        """
        Test assignment of mboxDirs attribute, mixed valid and invalid entries.
        """
        mbox = MboxConfig()
        self.assertEqual(None, mbox.mboxDirs)
        self.failUnlessAssignRaises(ValueError, mbox, "mboxDirs", [MboxDir(), MboxFile()])
        self.assertEqual(None, mbox.mboxDirs)

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
        Test comparison of two identical objects, all attributes None.
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig()
        self.assertEqual(mbox1, mbox2)
        self.assertTrue(mbox1 == mbox2)
        self.assertTrue(not mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(mbox1 >= mbox2)
        self.assertTrue(not mbox1 != mbox2)

    def testComparison_002(self):
        """
        Test comparison of two identical objects, all attributes non-None, lists None.
        """
        mbox1 = MboxConfig("daily", "gzip", None, None)
        mbox2 = MboxConfig("daily", "gzip", None, None)
        self.assertEqual(mbox1, mbox2)
        self.assertTrue(mbox1 == mbox2)
        self.assertTrue(not mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(mbox1 >= mbox2)
        self.assertTrue(not mbox1 != mbox2)

    def testComparison_003(self):
        """
        Test comparison of two identical objects, all attributes non-None, lists empty.
        """
        mbox1 = MboxConfig("daily", "gzip", [], [])
        mbox2 = MboxConfig("daily", "gzip", [], [])
        self.assertEqual(mbox1, mbox2)
        self.assertTrue(mbox1 == mbox2)
        self.assertTrue(not mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(mbox1 >= mbox2)
        self.assertTrue(not mbox1 != mbox2)

    def testComparison_004(self):
        """
        Test comparison of two identical objects, all attributes non-None, lists non-empty.
        """
        mbox1 = MboxConfig("daily", "gzip", [MboxFile()], [MboxDir()])
        mbox2 = MboxConfig("daily", "gzip", [MboxFile()], [MboxDir()])
        self.assertEqual(mbox1, mbox2)
        self.assertTrue(mbox1 == mbox2)
        self.assertTrue(not mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(mbox1 >= mbox2)
        self.assertTrue(not mbox1 != mbox2)

    def testComparison_005(self):
        """
        Test comparison of two differing objects, collectMode differs (one None).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(collectMode="daily")
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_006(self):
        """
        Test comparison of two differing objects, collectMode differs.
        """
        mbox1 = MboxConfig("daily", "gzip", [MboxFile()])
        mbox2 = MboxConfig("weekly", "gzip", [MboxFile()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_007(self):
        """
        Test comparison of two differing objects, compressMode differs (one None).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(compressMode="bzip2")
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_008(self):
        """
        Test comparison of two differing objects, compressMode differs.
        """
        mbox1 = MboxConfig("daily", "bzip2", [MboxFile()])
        mbox2 = MboxConfig("daily", "gzip", [MboxFile()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_009(self):
        """
        Test comparison of two differing objects, mboxFiles differs (one None, one empty).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(mboxFiles=[])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_010(self):
        """
        Test comparison of two differing objects, mboxFiles differs (one None, one not empty).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(mboxFiles=[MboxFile()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_011(self):
        """
        Test comparison of two differing objects, mboxFiles differs (one empty, one not empty).
        """
        mbox1 = MboxConfig("daily", "gzip", [], None)
        mbox2 = MboxConfig("daily", "gzip", [MboxFile()], None)
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_012(self):
        """
        Test comparison of two differing objects, mboxFiles differs (both not empty).
        """
        mbox1 = MboxConfig("daily", "gzip", [MboxFile()], None)
        mbox2 = MboxConfig("daily", "gzip", [MboxFile(), MboxFile()], None)
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_013(self):
        """
        Test comparison of two differing objects, mboxDirs differs (one None, one empty).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(mboxDirs=[])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_014(self):
        """
        Test comparison of two differing objects, mboxDirs differs (one None, one not empty).
        """
        mbox1 = MboxConfig()
        mbox2 = MboxConfig(mboxDirs=[MboxDir()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_015(self):
        """
        Test comparison of two differing objects, mboxDirs differs (one empty, one not empty).
        """
        mbox1 = MboxConfig("daily", "gzip", None, [])
        mbox2 = MboxConfig("daily", "gzip", None, [MboxDir()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)

    def testComparison_016(self):
        """
        Test comparison of two differing objects, mboxDirs differs (both not empty).
        """
        mbox1 = MboxConfig("daily", "gzip", None, [MboxDir()])
        mbox2 = MboxConfig("daily", "gzip", None, [MboxDir(), MboxDir()])
        self.assertNotEqual(mbox1, mbox2)
        self.assertTrue(not mbox1 == mbox2)
        self.assertTrue(mbox1 < mbox2)
        self.assertTrue(mbox1 <= mbox2)
        self.assertTrue(not mbox1 > mbox2)
        self.assertTrue(not mbox1 >= mbox2)
        self.assertTrue(mbox1 != mbox2)


########################
# TestLocalConfig class
########################


class TestLocalConfig(unittest.TestCase):
    """Tests for the LocalConfig class."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        try:
            self.resources = findResources(RESOURCES, DATA_DIRS)
        except Exception as e:
            self.fail(e)

    def tearDown(self):
        pass

    ##################
    # Utility methods
    ##################

    def failUnlessAssignRaises(self, exception, obj, prop, value):
        """Equivalent of :any:`failUnlessRaises`, but used for property assignments instead."""
        failUnlessAssignRaises(self, exception, obj, prop, value)

    def validateAddConfig(self, origConfig):
        """
        Validates that document dumped from ``LocalConfig.addConfig`` results in
        identical object.

        We dump a document containing just the mbox configuration, and then
        make sure that if we push that document back into the ``LocalConfig``
        object, that the resulting object matches the original.

        The ``self.failUnlessEqual`` method is used for the validation, so if the
        method call returns normally, everything is OK.

        Args:
           origConfig: Original configuration
        """
        (xmlDom, parentNode) = createOutputDom()
        origConfig.addConfig(xmlDom, parentNode)
        xmlData = serializeDom(xmlDom)
        newConfig = LocalConfig(xmlData=xmlData, validate=False)
        self.assertEqual(origConfig, newConfig)

    ############################
    # Test __repr__ and __str__
    ############################

    def testStringFuncs_001(self):
        """
        Just make sure that the string functions don't have errors (i.e. bad variable names).
        """
        obj = LocalConfig()
        obj.__repr__()
        obj.__str__()

    #####################################################
    # Test basic constructor and attribute functionality
    #####################################################

    def testConstructor_001(self):
        """
        Test empty constructor, validate=False.
        """
        config = LocalConfig(validate=False)
        self.assertEqual(None, config.mbox)

    def testConstructor_002(self):
        """
        Test empty constructor, validate=True.
        """
        config = LocalConfig(validate=True)
        self.assertEqual(None, config.mbox)

    def testConstructor_003(self):
        """
        Test with empty config document as both data and file, validate=False.
        """
        path = self.resources["mbox.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, xmlPath=path, validate=False)

    def testConstructor_004(self):
        """
        Test assignment of mbox attribute, None value.
        """
        config = LocalConfig()
        config.mbox = None
        self.assertEqual(None, config.mbox)

    def testConstructor_005(self):
        """
        Test assignment of mbox attribute, valid value.
        """
        config = LocalConfig()
        config.mbox = MboxConfig()
        self.assertEqual(MboxConfig(), config.mbox)

    def testConstructor_006(self):
        """
        Test assignment of mbox attribute, invalid value (not MboxConfig).
        """
        config = LocalConfig()
        self.failUnlessAssignRaises(ValueError, config, "mbox", "STRING!")

    ############################
    # Test comparison operators
    ############################

    def testComparison_001(self):
        """
        Test comparison of two identical objects, all attributes None.
        """
        config1 = LocalConfig()
        config2 = LocalConfig()
        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_002(self):
        """
        Test comparison of two identical objects, all attributes non-None.
        """
        config1 = LocalConfig()
        config1.mbox = MboxConfig()

        config2 = LocalConfig()
        config2.mbox = MboxConfig()

        self.assertEqual(config1, config2)
        self.assertTrue(config1 == config2)
        self.assertTrue(not config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(config1 >= config2)
        self.assertTrue(not config1 != config2)

    def testComparison_003(self):
        """
        Test comparison of two differing objects, mbox differs (one None).
        """
        config1 = LocalConfig()
        config2 = LocalConfig()
        config2.mbox = MboxConfig()
        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    def testComparison_004(self):
        """
        Test comparison of two differing objects, mbox differs.
        """
        config1 = LocalConfig()
        config1.mbox = MboxConfig(collectMode="daily")

        config2 = LocalConfig()
        config2.mbox = MboxConfig(collectMode="weekly")

        self.assertNotEqual(config1, config2)
        self.assertTrue(not config1 == config2)
        self.assertTrue(config1 < config2)
        self.assertTrue(config1 <= config2)
        self.assertTrue(not config1 > config2)
        self.assertTrue(not config1 >= config2)
        self.assertTrue(config1 != config2)

    ######################
    # Test validate logic
    ######################

    def testValidate_001(self):
        """
        Test validate on a None mbox section.
        """
        config = LocalConfig()
        config.mbox = None
        self.assertRaises(ValueError, config.validate)

    def testValidate_002(self):
        """
        Test validate on an empty mbox section.
        """
        config = LocalConfig()
        config.mbox = MboxConfig()
        self.assertRaises(ValueError, config.validate)

    def testValidate_003(self):
        """
        Test validate on a non-empty mbox section, mboxFiles=None and mboxDirs=None.
        """
        config = LocalConfig()
        config.mbox = MboxConfig("weekly", "gzip", None, None)
        self.assertRaises(ValueError, config.validate)

    def testValidate_004(self):
        """
        Test validate on a non-empty mbox section, mboxFiles=[] and mboxDirs=[].
        """
        config = LocalConfig()
        config.mbox = MboxConfig("weekly", "gzip", [], [])
        self.assertRaises(ValueError, config.validate)

    def testValidate_005(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        defaults set, no values on files.
        """
        mboxFiles = [MboxFile(absolutePath="/one"), MboxFile(absolutePath="/two")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_006(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        defaults set, no values on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/one"), MboxDir(absolutePath="/two")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_007(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        no defaults set, no values on files.
        """
        mboxFiles = [MboxFile(absolutePath="/one"), MboxFile(absolutePath="/two")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        self.assertRaises(ValueError, config.validate)

    def testValidate_008(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        no defaults set, no values on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/one"), MboxDir(absolutePath="/two")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        self.assertRaises(ValueError, config.validate)

    def testValidate_009(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        no defaults set, both values on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", collectMode="weekly", compressMode="gzip")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_010(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        no defaults set, both values on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", collectMode="weekly", compressMode="gzip")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_011(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        collectMode only on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", collectMode="weekly")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_012(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        collectMode only on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", collectMode="weekly")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_013(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        compressMode only on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "weekly"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_014(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        compressMode only on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "weekly"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_015(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        compressMode default and on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_016(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        compressMode default and on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_017(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        collectMode default and on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", collectMode="daily")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_018(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        collectMode default and on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", collectMode="daily")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    def testValidate_019(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxFiles,
        collectMode and compressMode default and on files.
        """
        mboxFiles = [MboxFile(absolutePath="/two", collectMode="daily", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = mboxFiles
        config.mbox.mboxDirs = None
        config.validate()

    def testValidate_020(self):
        """
        Test validate on a non-empty mbox section, non-empty mboxDirs,
        collectMode and compressMode default and on directories.
        """
        mboxDirs = [MboxDir(absolutePath="/two", collectMode="daily", compressMode="bzip2")]
        config = LocalConfig()
        config.mbox = MboxConfig()
        config.mbox.collectMode = "daily"
        config.mbox.compressMode = "gzip"
        config.mbox.mboxFiles = None
        config.mbox.mboxDirs = mboxDirs
        config.validate()

    ############################
    # Test parsing of documents
    ############################

    def testParse_001(self):
        """
        Parse empty config document.
        """
        path = self.resources["mbox.conf.1"]
        with open(path) as f:
            contents = f.read()
        self.assertRaises(ValueError, LocalConfig, xmlPath=path, validate=True)
        self.assertRaises(ValueError, LocalConfig, xmlData=contents, validate=True)
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertEqual(None, config.mbox)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertEqual(None, config.mbox)

    def testParse_002(self):
        """
        Parse config document with default modes, one collect file and one collect dir.
        """
        mboxFiles = [
            MboxFile(absolutePath="/home/joebob/mail/cedar-backup-users"),
        ]
        mboxDirs = [
            MboxDir(absolutePath="/home/billiejoe/mail"),
        ]
        path = self.resources["mbox.conf.2"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual("daily", config.mbox.collectMode)
        self.assertEqual("gzip", config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual("daily", config.mbox.collectMode)
        self.assertEqual("gzip", config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)

    def testParse_003(self):
        """
        Parse config document with no default modes, one collect file and one collect dir.
        """
        mboxFiles = [
            MboxFile(absolutePath="/home/joebob/mail/cedar-backup-users", collectMode="daily", compressMode="gzip"),
        ]
        mboxDirs = [
            MboxDir(absolutePath="/home/billiejoe/mail", collectMode="weekly", compressMode="bzip2"),
        ]
        path = self.resources["mbox.conf.3"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual(None, config.mbox.collectMode)
        self.assertEqual(None, config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual(None, config.mbox.collectMode)
        self.assertEqual(None, config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)

    def testParse_004(self):
        """
        Parse config document with default modes, several files with
        various overrides and exclusions.
        """
        mboxFiles = []
        mboxFile = MboxFile(absolutePath="/home/jimbo/mail/cedar-backup-users")
        mboxFiles.append(mboxFile)
        mboxFile = MboxFile(absolutePath="/home/joebob/mail/cedar-backup-users", collectMode="daily", compressMode="gzip")
        mboxFiles.append(mboxFile)
        mboxDirs = []
        mboxDir = MboxDir(absolutePath="/home/frank/mail/cedar-backup-users")
        mboxDirs.append(mboxDir)
        mboxDir = MboxDir(absolutePath="/home/jimbob/mail", compressMode="bzip2", relativeExcludePaths=["logomachy-devel"])
        mboxDirs.append(mboxDir)
        mboxDir = MboxDir(
            absolutePath="/home/billiejoe/mail", collectMode="weekly", compressMode="bzip2", excludePatterns=[".*SPAM.*"]
        )
        mboxDirs.append(mboxDir)
        mboxDir = MboxDir(
            absolutePath="/home/billybob/mail",
            relativeExcludePaths=["debian-devel", "debian-python"],
            excludePatterns=[".*SPAM.*", ".*JUNK.*"],
        )
        mboxDirs.append(mboxDir)
        path = self.resources["mbox.conf.4"]
        with open(path) as f:
            contents = f.read()
        config = LocalConfig(xmlPath=path, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual("incr", config.mbox.collectMode)
        self.assertEqual("none", config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)
        config = LocalConfig(xmlData=contents, validate=False)
        self.assertNotEqual(None, config.mbox)
        self.assertEqual("incr", config.mbox.collectMode)
        self.assertEqual("none", config.mbox.compressMode)
        self.assertEqual(mboxFiles, config.mbox.mboxFiles)
        self.assertEqual(mboxDirs, config.mbox.mboxDirs)

    ###################
    # Test addConfig()
    ###################

    def testAddConfig_001(self):
        """
        Test with empty config document.
        """
        mbox = MboxConfig()
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_002(self):
        """
        Test with defaults set, single mbox file with no optional values.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_003(self):
        """
        Test with defaults set, single mbox directory with no optional values.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_004(self):
        """
        Test with defaults set, single mbox file with collectMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="incr"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_005(self):
        """
        Test with defaults set, single mbox directory with collectMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="incr"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_006(self):
        """
        Test with defaults set, single mbox file with compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", compressMode="bzip2"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_007(self):
        """
        Test with defaults set, single mbox directory with compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", compressMode="bzip2"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_008(self):
        """
        Test with defaults set, single mbox file with collectMode and compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="weekly", compressMode="bzip2"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_009(self):
        """
        Test with defaults set, single mbox directory with collectMode and compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="weekly", compressMode="bzip2"))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_010(self):
        """
        Test with no defaults set, single mbox file with collectMode and compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="weekly", compressMode="bzip2"))
        mbox = MboxConfig(mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_011(self):
        """
        Test with no defaults set, single mbox directory with collectMode and compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="weekly", compressMode="bzip2"))
        mbox = MboxConfig(mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_012(self):
        """
        Test with compressMode set, single mbox file with collectMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="weekly"))
        mbox = MboxConfig(compressMode="gzip", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_013(self):
        """
        Test with compressMode set, single mbox directory with collectMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="weekly"))
        mbox = MboxConfig(compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_014(self):
        """
        Test with collectMode set, single mbox file with compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", compressMode="gzip"))
        mbox = MboxConfig(collectMode="weekly", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_015(self):
        """
        Test with collectMode set, single mbox directory with compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", compressMode="gzip"))
        mbox = MboxConfig(collectMode="weekly", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_016(self):
        """
        Test with compressMode set, single mbox file with collectMode and compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="incr", compressMode="gzip"))
        mbox = MboxConfig(compressMode="bzip2", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_017(self):
        """
        Test with compressMode set, single mbox directory with collectMode and compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="incr", compressMode="gzip"))
        mbox = MboxConfig(compressMode="bzip2", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_018(self):
        """
        Test with collectMode set, single mbox file with collectMode and compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path", collectMode="weekly", compressMode="gzip"))
        mbox = MboxConfig(collectMode="incr", mboxFiles=mboxFiles)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_019(self):
        """
        Test with collectMode set, single mbox directory with collectMode and compressMode set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", collectMode="weekly", compressMode="gzip"))
        mbox = MboxConfig(collectMode="incr", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_020(self):
        """
        Test with defaults set, single mbox directory with relativeExcludePaths set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", relativeExcludePaths=["one", "two"]))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_021(self):
        """
        Test with defaults set, single mbox directory with excludePatterns set.
        """
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path", excludePatterns=["one", "two"]))
        mbox = MboxConfig(collectMode="daily", compressMode="gzip", mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)

    def testAddConfig_022(self):
        """
        Test with defaults set, multiple mbox files and directories with collectMode and compressMode set.
        """
        mboxFiles = []
        mboxFiles.append(MboxFile(absolutePath="/path1", collectMode="daily", compressMode="gzip"))
        mboxFiles.append(MboxFile(absolutePath="/path2", collectMode="weekly", compressMode="gzip"))
        mboxFiles.append(MboxFile(absolutePath="/path3", collectMode="incr", compressMode="gzip"))
        mboxDirs = []
        mboxDirs.append(MboxDir(absolutePath="/path1", collectMode="daily", compressMode="bzip2"))
        mboxDirs.append(MboxDir(absolutePath="/path2", collectMode="weekly", compressMode="bzip2"))
        mboxDirs.append(MboxDir(absolutePath="/path3", collectMode="incr", compressMode="bzip2"))
        mbox = MboxConfig(collectMode="incr", compressMode="bzip2", mboxFiles=mboxFiles, mboxDirs=mboxDirs)
        config = LocalConfig()
        config.mbox = mbox
        self.validateAddConfig(config)
